//
// This code is part of https://en.onboarding.grapecitydev.com/documents-api-pdf/demos.
// Copyright (c) GrapeCity, Inc. All rights reserved.
//
using System;
using System.IO;
using System.Drawing;
using System.Collections.Generic;
using GrapeCity.Documents.Pdf;
using GrapeCity.Documents.Pdf.Recognition;
using GrapeCity.Documents.Text;
using GrapeCity.Documents.Common;
using System.Text;
using System.Linq;
using System.Diagnostics;
using GrapeCity.Documents.Excel;
using System.Threading;

namespace GcPdfWeb.Samples
{
    // Extract data from a table.
    public class GetTableData
    {
        public void GetTablePage(string fileName)
        {
            const float DPI = 72;
            using (var fs = File.OpenRead(Path.Combine("Resources", "PDFs", "zugferd-invoice.pdf")))
            {
                var pdfDoc = new GcPdfDocument();
                pdfDoc.Load(fs);

                #region Extract tabular data

                // The approx table bounds:
                var tableBounds = new RectangleF(0, 2.5f * DPI, 8.5f * DPI, 3.75f * DPI);

                // TableExtractOptions allow to fine-tune table recognition accounting for
                // specifics of the table formatting:
                var tableExtrctOpt = new TableExtractOptions();
                var GetMinimumDistanceBetweenRows = tableExtrctOpt.GetMinimumDistanceBetweenRows;

                // In this particular case, we slightly increase the minimum distance between rows
                // to make sure cells with wrapped text are not mistaken for two cells:
                tableExtrctOpt.GetMinimumDistanceBetweenRows = (list) => {
                    var res = GetMinimumDistanceBetweenRows(list);
                    return res * 1.2f;
                };

                // CSV: list to keep table data from all pages:
                var data = new List<List<string>>();
                for (int i = 0; i < pdfDoc.Pages.Count; ++i)
                {
                    // Get the table at the specified bounds:
                    var itable = pdfDoc.Pages[i].GetTable(tableBounds, tableExtrctOpt);
                    if (itable != null)
                    {
                        for (int row = 0; row < itable.Rows.Count; ++row)
                        {
                            // CSV: add next data row ignoring headers:
                            if (row > 0)
                                data.Add(new List<string>());

                            for (int col = 0; col < itable.Cols.Count; ++col)
                            {
                                var cell = itable.GetCell(row, col);
                                if (cell == null && row > 0)
                                    data.Last().Add("");
                                else
                                {
                                    if (cell != null && row > 0)
                                        data.Last().Add($"\"{cell.Text}\"");
                                }
                            }
                        }
                    }
                }
                Console.WriteLine("Extraction successful");
                #endregion

                #region Export extracted data to CSV
                Encoding.RegisterProvider(CodePagesEncodingProvider.Instance); // needed to encode non-ASCII chars in data
                File.Delete(fileName);
                File.AppendAllLines(
                    fileName,
                    data.Where(l_ => l_.Any(s_ => !string.IsNullOrEmpty(s_))).Select(d_ => string.Join(',', d_)),
                    Encoding.GetEncoding(1252));
                Console.WriteLine($"Export successful");
                #endregion

                #region Formatting Exported data in XLSX
                var workbook = new GrapeCity.Documents.Excel.Workbook();
                workbook.Open($@"{fileName}", OpenFileFormat.Csv);

                IWorksheet worksheet = workbook.Worksheets[0];
                IRange range = worksheet.Range["A2:E10"];

                // wrapping cell content
                range.WrapText = true;

                // styling column names 
                worksheet.Range["A1"].EntireRow.Font.Bold = true;

                // auto-sizing range
                worksheet.Range["A1:E10"].AutoFit();

                // aligning cell content
                worksheet.Range["A1:E10"].HorizontalAlignment = HorizontalAlignment.Center;
                worksheet.Range["A1:E10"].VerticalAlignment = VerticalAlignment.Center;

                // applying conditional format on UnitPrice
                IColorScale twoColorScaleRule = worksheet.Range["E2:E10"].FormatConditions.AddColorScale(ColorScaleType.TwoColorScale);

                twoColorScaleRule.ColorScaleCriteria[0].Type = ConditionValueTypes.LowestValue;
                twoColorScaleRule.ColorScaleCriteria[0].FormatColor.Color = Color.FromArgb(255, 229, 229);

                twoColorScaleRule.ColorScaleCriteria[1].Type = ConditionValueTypes.HighestValue;
                twoColorScaleRule.ColorScaleCriteria[1].FormatColor.Color = Color.FromArgb(255, 20, 20);

                Thread.Sleep(1000);

                workbook.Save("ExtractedData_Formatted.xlsx");
                Console.WriteLine($"Formatting successful");

                ProcessStartInfo psi = new ProcessStartInfo() { FileName = "ExtractedData_Formatted.xlsx", UseShellExecute = true };
                Process.Start(psi);
                #endregion
            }
        }
    }
}
